/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_svsnick.c: Sets a users nick to random one when the nickname is
 *               registered and user did not identify within 30 seconds.
 *
 *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 *  Copyright (C) 1996-2002 Hybrid Development Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_svsnick.c,v 1.109 2004/02/10 14:46:00 HaKer4o Exp $
 */

#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "hash.h"
#include "fdlist.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "s_conf.h"
#include "s_stats.h"
#include "s_user.h"
#include "hash.h"
#include "whowas.h"
#include "s_serv.h"
#include "send.h"
#include "channel.h"
#include "s_log.h"
#include "resv.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "common.h"
#include "packet.h"


static void m_svsnick(struct Client *, struct Client *, int, char **);

static int check_clean_nick(struct Client *, struct Client *, char *, char *, char *);
static int clean_nick_name(char *);

struct Message svsnick_msgtab = {
	"SVSNICK", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_ignore, m_svsnick, m_svsnick, m_svsnick}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&svsnick_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&svsnick_msgtab);
}

const char *_version = "$Revision: 1.1 $";
#endif

/*
 * m_svsnick()
 *
 *     parv[0] = sender prefix
 *     parv[1] = target nickname
 *     parv[2] = new nickname
 */
static void
m_svsnick(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	char nick[NICKLEN];
	struct Client *target_p;
	struct Client *dest_p;

	if (!IsServer(source_p) && !IsAdmin(source_p)) {
	    return;
	}
	
	if(parc < 3 || BadPtr(parv[2]))
	{
  	  sendto_wallops_flags(UMODE_WALLOP, &me,
	     "No nickname given via SVSNICK from %s for %s", parv[0], parv[1]);
		return;
	}

	/* terminate nick to NICKLEN */
	strlcpy(nick, parv[2], sizeof(nick));

	/* check the nickname is ok */
	if(!clean_nick_name(nick))
	{
  	  sendto_wallops_flags(UMODE_WALLOP, &me,
	     "Bogus nickname via SVSNICK from %s for %s : %s", parv[0], parv[1], parv[2]);
      return;
	}

	if(find_nick_resv(nick))
	{
  	  sendto_wallops_flags(UMODE_WALLOP, &me,
	     "Unavailable nickname via SVSNICK from %s for %s : %s", parv[0], parv[1], parv[2]);
		return;
	}
	
	if(!(dest_p = find_client(parv[1]))) {
  	  sendto_wallops_flags(UMODE_WALLOP, &me,
	     "Nickname (%s) not found via SVSNICK",parv[1]);
		return;
	}
	if (MyClient(dest_p)) {
    	    ilog(L_ALL,"SVSNICK LOCAL %s %s",parv[1],parv[2]);	
  	  sendto_wallops_flags(UMODE_WALLOP, &me,
	     "SVSNICK [ %s ] -> [ %s ] by %s",parv[1],parv[2],parv[0]);

	    change_local_nick(&me, dest_p, nick);
	    }
	else {
    	    ilog(L_ALL,"SVSNICK REMOTE %s %s",parv[1],parv[2]);	
  	  sendto_wallops_flags(UMODE_WALLOP, &me,
	     "SVSNICK [ %s ] -> [ %s ] by %s",parv[1],parv[2],parv[0]);
	    sendto_one(dest_p->servptr,":%s SVSNICK %s %s", me.name,
	                                      parv[1], nick);
	}
}
/* 
 * check_clean_nick()
 * 
 * input	- pointer to source
 *		- nickname
 *		- truncated nickname
 *		- origin of client
 * output	- none
 * side effects - if nickname is erroneous, or a different length to
 *                truncated nickname, return 1
 */
static int
check_clean_nick(struct Client *client_p, struct Client *source_p,
		 char *nick, char *newnick, char *server)
{
	/* the old code did some wacky stuff here, if the nick is invalid, kill it
	 * and dont bother messing at all
	 */

	/*
	 * Zero length nicks are bad too..this shouldn't happen but..
	 */

	if(!clean_nick_name(nick) || strcmp(nick, newnick) || nick[0] == '\0')
	{
		ServerStats->is_kill++;
		sendto_realops_flags(UMODE_DEBUG, L_ALL,
				     "Bad Nick: %s From: %s(via %s)", nick, server, client_p->name);

		sendto_one(client_p, ":%s KILL %s :%s (Bad Nickname)", me.name, newnick, me.name);

		/* bad nick change */
		if(source_p != client_p)
		{
			kill_client_serv_butone(client_p, source_p, "%s (Bad Nickname)", me.name);
			source_p->flags |= FLAGS_KILLED;
			exit_client(client_p, source_p, &me, "Bad Nickname");
		}

		return 1;
	}

	return 0;
}

/* clean_nick_name()
 *
 * input	- nickname
 * output	- none
 * side effects - walks through the nickname, returning 0 if erroneous
 */
static int
clean_nick_name(char *nick)
{
	assert(nick);
	if(nick == NULL)
		return 0;

	/* nicks cant start with a digit or -, and must have a length */
	if(*nick == '-' || IsDigit(*nick) || !*nick)
		return 0;

	for (; *nick; nick++)
	{
		if(!IsNickChar(*nick))
			return 0;
	}

	return 1;
}

