/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  m_part.c: Parts a user from a channel.
 *
 *  Copyright (C) 1990 Jarkko Oikarinen and University of Oulu, Co Center
 *  Copyright (C) 1996-2002 Hybrid Development Team
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: m_svspart.c,v 1.80.3.3 2004/02/07 15:44:42 MaskBg Exp $
 */

#include "stdinc.h"
#include "tools.h"
#include "handlers.h"
#include "channel.h"
#include "channel_mode.h"
#include "client.h"
#include "common.h"
#include "hash.h"
#include "irc_string.h"
#include "ircd.h"
#include "numeric.h"
#include "send.h"
#include "s_serv.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"
#include "s_conf.h"
#include "packet.h"

static void m_svspart(struct Client *, struct Client *, int, char **);
void check_spambot_warning(struct Client *source_p, const char *name);

struct Message svspart_msgtab = {
	"SVSPART", 0, 0, 3, 0, MFLG_SLOW, 0,
	{m_unregistered, m_svspart, m_svspart, m_svspart}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
	mod_add_cmd(&svspart_msgtab);
}

void
_moddeinit(void)
{
	mod_del_cmd(&svspart_msgtab);
}
const char *_version = "$Revision: 1.1.8 $";
#endif

static void part_one_client(struct Client *client_p,
			    struct Client *source_p, char *name, char *reason);


/*
** m_svspart
**      parv[0] = sender prefix
**      parv[1] = channel
**      parv[2] = reason
*/
static void
m_svspart(struct Client *client_p, struct Client *source_p, int parc, char *parv[])
{
	char *p, *name;
	char reason[TOPICLEN + 1];
	struct Client *dest_p = NULL;

	if (!IsServer(source_p) && !IsAdmin(source_p)) {
		sendto_one(source_p, form_str(ERR_RESTRICTED), me.name, source_p->name);
		return;
	
	}

	if(EmptyString(parv[1]) || EmptyString(parv[2]))
	{
		sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, parv[0], "SVSPART");
		return;
	}
	if(!IsServer(source_p) && !IsAdmin(source_p))
	{
		sendto_realops_flags(UMODE_ALL, L_ADMIN,
				     "DENIED ACCESS to SVSPART for %s !", source_p->name);
		ilog(L_ALL, "DENIED ACCESS to SVSPART for %s !", source_p->name);
		return;
	}
	if(!(dest_p = find_person(parv[1])))
	{
		sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, parv[0], parv[1]);
		return;
	}

	if(!MyClient(dest_p))
	{
		sendto_realops_flags(UMODE_ALL, L_ADMIN, "Forwarding SVSPART from %s for %s/%s",
				     parv[0], parv[1], parv[2]);
		ilog(L_ALL, "Forwarding SVSPART from %s for %s/%s", parv[0], parv[1], parv[2]);
		sendto_one(dest_p->servptr, ":%s SVSPART %s %s %s", parv[0],
			   parv[1], parv[2], (parv[3]) ? parv[3] : NULL);
		return;
	}
	else
	{
		sendto_realops_flags(UMODE_ALL, L_ADMIN, "Received SVSPART from %s for %s/%s",
				     parv[0], parv[1], parv[2]);
		ilog(L_ALL, "Received SVSPART from %s for %s/%s", parv[0], parv[1], parv[2]);
	}

	reason[0] = '\0';

	if(parc > 3)
		strlcpy(reason, parv[3], sizeof(reason));

	name = strtoken(&p, parv[2], ",");

	/* Finish the flood grace period... */
	if(MyClient(dest_p) && !IsFloodDone(dest_p))
		flood_endgrace(dest_p);

	while (name)
	{
		part_one_client(client_p, dest_p, name, reason);
		name = strtoken(&p, (char *) NULL, ",");
	}
	return;
}

/*
 * part_one_client
 *
 * inputs	- pointer to server
 * 		- pointer to source client to remove
 *		- char pointer of name of channel to remove from
 * output	- none
 * side effects	- remove ONE client given the channel name 
 */
static void
part_one_client(struct Client *client_p, struct Client *source_p, char *name, char *reason)
{
	struct Channel *chptr;
	struct membership *msptr;

	if((chptr = hash_find_channel(name)) == NULL)
	{
		sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL), me.name, source_p->name, name);
		return;
	}

	msptr = find_channel_membership(chptr, source_p);
	if(msptr == NULL)
	{
		sendto_one(source_p, form_str(ERR_NOTONCHANNEL), me.name, source_p->name, name);
		return;
	}
	if(MyConnect(source_p) && !IsOper(source_p) && !IsExemptSpambot(source_p))
		check_spambot_warning(source_p, NULL);

	/*
	 *  Remove user from the old channel (if any)
	 *  only allow /part reasons in -m chans
	 */
	if(reason[0] && (is_chanop(msptr) || !MyConnect(source_p) ||
			 ((can_send(chptr, source_p, msptr) > 0 &&
			   (source_p->firsttime + ConfigFileEntry.anti_spam_exit_message_time)
			   < CurrentTime))))
	{
		sendto_server(&me /* client_p */, chptr, NOCAPS, NOCAPS,
			      ":%s PART %s :%s", source_p->name, chptr->chname, reason);

		sendto_channel_local(ALL_MEMBERS,
				     chptr, ":%s!%s@%s PART %s :%s",
				     source_p->name,
				     source_p->username, source_p->host, chptr->chname, reason);
	}
	else
	{
		sendto_server(&me /* client_p */, chptr, NOCAPS, NOCAPS,
			      ":%s PART %s", source_p->name, chptr->chname);

		sendto_channel_local(ALL_MEMBERS,
				     chptr, ":%s!%s@%s PART %s",
				     source_p->name,
				     source_p->username, source_p->host, chptr->chname);
	}
	del_nokick_chan(source_p, name);
	remove_user_from_channel(msptr);
}
